package
{
	import com.greensock.TweenMax;
	import com.greensock.easing.Linear;
	
	import flashx.textLayout.formats.Float;
	
	import mx.utils.StringUtil;
	
	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxPath;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;
	
	
	public class Game
	{
		public var grid:Vector.<Vector.<int>>;
		
		public var gridsize:int=15;
		
		public var lastx:int=-1;
		public var lasty:int=-1;
		
		function Game(desc:String="")
		{
			grid = new Vector.<Vector.<int>>();
			
			if (desc=="")
				return;
			
			var ar:Array = desc.split("\n");
			for (var i:int=0;i<ar.length;i++)
			{
				var row:Vector.<int> = new Vector.<int>();
				var s:String = ar[i];				
				//s = mx.utils.StringUtil.trim(s);
				for (var j:int=0;j<s.length;j++)
				{
					var c:String = s.charAt(j);
					if (c==" ")
						c="-1";
					var num:int = int(c);
					row.push(num);
				}
				grid.push(row);
			}
		}
		
		public static function Generate(width:int,height:int,maxnum:int,explosions:int):Game
		{
			var g:Game = new Game();
			
			var i:int=0;
			var j:int=0;
			
			g.grid = new Vector.<Vector.<int>>();
			for (i=0;i<height;i++)
			{
				var r:Vector.<int> = new Vector.<int>();
				for (j=0;j<width;j++)
				{
					r.push(-1);
				}
				g.grid.push(r);
			}
			
			//place top tile
			var rx:int = Math.random()*(2);
			var ry:int = Math.random()*(2);
			g.grid[width/2+rx][width/2+ry]=maxnum;
			
			for (i=0;i<explosions;i++)
			{
				g.TryExplode();
			}
			
			return g;
		}
		
		public function CanExplode(x:int,y:int):Boolean
		{
			if (x<=0||y<=0||x>=grid[0].length-1||y>=grid.length-1)
				return false;
			
			if (grid[y][x]<1)
				return false;
			return grid[y][0]<0 && grid[0][x]<0 && grid[y][grid[0].length-1]<0 && grid[grid.length-1][x]<0;
		}
		
		public function Explode(x:int,y:int):void
		{
			var i:int;
			var j:int;
			
			
			grid[y][x]--;
			
			for (i=0;i<y;i++)
				grid[i][x]=grid[i+1][x];
			
			for (i=0;i<x;i++)
				grid[y][i]=grid[y][i+1];
			
			for (i=grid.length-1;i>y;i--)
				grid[i][x]=grid[i-1][x];
			
			for (i=grid[0].length-1;i>x;i--)
				grid[y][i]=grid[y][i-1];
			
			
		}
		
		public function TryExplode():void
		{
			var tilecount = grid.length*grid[0].length;
			var r:int = Math.random()*tilecount;
			for (var offset:int=0;offset<tilecount;offset++)
			{
				var ti:int = (offset+r) %tilecount;
				var tx:int = ti%grid.length;
				var ty:int = ti/grid.length;
				if (grid[ty][tx]>0 && CanExplode(tx,ty))
				{
					Explode(tx,ty);
					return;
				}
			}
		}
		
		private function GetBlock(x:int,y:int):int
		{
			if (x<0 || y<0)
				return 0;
			if (y>=grid.length || x>=grid[0].length)
				return 0;
			return grid[y][x];
		}
		
		public function CanMove(x:int,y:int):Boolean
		{
			var tiles:Vector.<int>= new Vector.<int>();
			var center:int=GetBlock(x,y);
			
			var last:int=GetBlock(x,y);
			if (last==-1||last!=center)
				return false;
			last=GetBlock(x+1,y);
			if (last==-1||last!=center)
				return false;
			last=GetBlock(x-1,y);
			if (last==-1||last!=center)
				return false;
			last=GetBlock(x,y+1);
			if (last==-1||last!=center)
				return false;
			last=GetBlock(x,y-1);
			if (last==-1||last!=center)
				return false;
			
			return true;
		}
		
		public function TryMove(x:int, y:int):Boolean
		{		
			if (CanMove(x,y)==false)
			{
				FlxG.play(TitleState.SndCantCompress);
				return false;
			}
			else
			{				
				FlxG.play(TitleState.SndCompress);
			}
			
			var i:int=0;
			
			grid[y][x]=grid[y][x]+1;
			
			for (i=y+1;i<grid.length-1;i++)
				grid[i][x]=grid[i+1][x];
			for (i=y-1;i>0;i--)
				grid[i][x]=grid[i-1][x];
			for (i=x+1;i<grid[0].length-1;i++)
				grid[y][i]=grid[y][i+1];
			for (i=x-1;i>0;i--)
				grid[y][i]=grid[y][i-1];
			
			grid[y][0]=-1;
			grid[0][x]=-1;
			grid[y][grid[0].length-1]=-1;
			grid[grid.length-1][x]=-1;
			
			lastx=x;
			lasty=y;
			
			return true;
			
		}		
		
		public function CheckWin():Boolean
		{			
			//if a single tile left
			var nonzerocount:int=0;
			for (var i:int=0;i<grid.length;i++)
			{
				for (var j:int=0;j<grid[i].length;j++)
				{
					if (grid[j][i]>=0)
						nonzerocount++;
				}
			}
			
			return nonzerocount==1;
		}
		
		public static var tilesize:int=10;
		
		public function GenSprites() : Vector.<FlxSprite>
		{
			var result:Vector.<FlxSprite> = new Vector.<FlxSprite>();
			for (var i:int=0;i<grid.length;i++)
			{
				for (var j:int=0;j<grid[0].length;j++)
				{
					if (grid[i][j]!=-1)
					{
						var s:FlxText = new FlxText(j*tilesize,i*tilesize,tilesize,grid[i][j].toString());
						s.color=FlxG.BLACK;
						TryTween(s,j,i);
						result.push(s);
					}		
					else
					{
						var s2:FlxText = new FlxText(j*tilesize,i*tilesize,tilesize,".");
						s2.color=0xff888888;
						result.push(s2);						
					}
				}
			}
			
			return result;
		}
		
		public var movedur:Number = 0.2;
		
		private function TryTween(s:FlxSprite,x:int,y:int):void
		{		
			if (lastx==-1||lasty==-1)
				return;
			
			if (x==lastx && y==lasty)
			{
				//s.visible=false;											
			}
			else if (x==lastx)
			{
				if (y<lasty)
					TweenMax.from(s,movedur,{x:s.x,y:s.y-tilesize});
				else
					TweenMax.from(s,movedur,{x:s.x,y:s.y+tilesize});					
			}
			else if (y==lasty)
			{
				if (x<lastx)
					TweenMax.from(s,movedur,{y:s.y,x:s.x-tilesize});
				else
					TweenMax.from(s,movedur,{y:s.y,x:s.x+tilesize});	
				
			}
			
		}
		public function ClearMoved():void
		{
			lastx=-1;
			lasty=-1;			
		}
		
		public function Copy():Game
		{
			var g:Game = new Game();
			for (var i:int=0;i<grid.length;i++)
			{
				var r:Vector.<int> = new Vector.<int>();
				for (var j:int=0;j<grid[i].length;j++)
				{
					r.push(grid[i][j]);
				}
				g.grid.push(r);
			}
			return g;
		}
	}
}